#include <global.h>
#include <array.h>
#include <string.h>

Array *ary_create(int allocate, void (*free_elem)(void *p))
{
  Array *ary = ALLOC(Array);
  if (allocate == 0) {
    ary->elems = NULL;
  } else {
    ary->elems = ALLOC_N(void *, allocate);
    memset(ary->elems, 0, sizeof(void *) * allocate);
  }
  ary->size = 0;
  ary->allocated = allocate;
  ary->free_elem = free_elem;

  return ary;
}

void ary_destroy(Array *ary)
{
  int i;
  if (ary->free_elem) {
    for (i = 0; i < ary->size; i++) {
      if (ary->elems[i])
        ary->free_elem(ary->elems[i]);
    }
  }
  free(ary->elems);
  free(ary);
}

void ary_set(Array *ary, int index, void *value)
{
  if (index >= ary->allocated) {
    ary->allocated = (index + 1)*2;
    REALLOC_N(ary->elems, void *, (ary->allocated));
    memset((&ary->elems[ary->size]), 0, sizeof(void *) * (ary->allocated - ary->size));
  }

  if (index >= ary->size)
    ary->size = index + 1;

  if (ary->free_elem && ary->elems[index])
    ary->free_elem(ary->elems[index]);

  ary->elems[index] = value;
}

void ary_append(Array *ary, void *value)
{
  ary_set(ary, ary->size, value);
}

void *ary_get(Array *ary, int index)
{
  if (index >= ary->size)
    return NULL;
  return ary->elems[index];
}

void ary_delete(Array *ary, int index)
{
  if (index >= ary->size)
    return;
  if (ary->free_elem && ary->elems[index])
    ary->free_elem(ary->elems[index]);
  ary->elems[index] = NULL;
  if (index == ary->size - 1)
    ary->size--;
}

void *ary_remove(Array *ary, int index)
{
  void *p;
  if (index >= ary->size)
    return NULL;
  p = ary->elems[index];
  ary->elems[index] = NULL;
  ary->size--;
  memmove(&ary->elems[index], &ary->elems[index + 1],
      sizeof(void *) *(ary->size - index));
  return p;
}
