#ifndef FRT_DOCUMENT_H
#define FRT_DOCUMENT_H

#include <global.h>
#include <hash.h>
#include <array.h>

/****************************************************************************
 *
 * DocField
 *
 ****************************************************************************/
enum {
  DF_STORE_YES = 0,
  DF_STORE_NO = 1,
  DF_STORE_COMPRESS = 2
};

enum {
  DF_INDEX_UNTOKENIZED = 0,
  DF_INDEX_TOKENIZED = 1,
  DF_INDEX_NO = 2,
  DF_INDEX_NO_NORMS = 3
};

enum {
  DF_TERM_VECTOR_NO = 0,
  DF_TERM_VECTOR_YES = 1,
  DF_TERM_VECTOR_WITH_POSITIONS = 2,
  DF_TERM_VECTOR_WITH_OFFSETS = 3,
  DF_TERM_VECTOR_WITH_POSITIONS_OFFSETS = 4
};

typedef struct DocField {
  char  *name;
  char  *data;
  int blen; /* used for binary fields to store data length */
  float boost;
  bool is_stored : 1;
  bool is_compressed : 1;
  bool is_indexed : 1;
  bool is_tokenized : 1;
  bool store_tv : 1;
  bool store_pos : 1;
  bool store_offset : 1;
  bool omit_norms : 1;
  bool is_binary : 1;
} DocField;

DocField *df_create(const char *name, char *data, int store, int index, int tv);
DocField *df_clone(DocField *self);
void df_set(DocField *df, const char *name, char *data, int store, int index, int tv);
void df_destroy(DocField *df);
void df_destroy_data(DocField *df);
void df_set_store(DocField *df, int store);
void df_set_index(DocField *df, int index);
void df_set_term_vector(DocField *df, int tv);
char *df_to_s(DocField *df);
DocField *df_create_binary(char *name, char *data, int blen, int store);

/****************************************************************************
 *
 * Document
 *
 ****************************************************************************/

typedef struct Document {
  HshTable *fields;
  Array **field_arr;
  int fcnt;
  DocField **df_arr;
  int dfcnt;
  float boost;
  free_ft free_data;
} Document;

Document *doc_create();
Document *doc_create_keep_data();
void doc_destroy(Document *doc);
void doc_add_field(Document *doc, DocField *df);
DocField *doc_get_field(Document *doc, const char *fname);
Array *doc_get_fields(Document *doc, const char *fname);
Array *doc_remove_fields(Document *doc, const char *fname);
DocField *doc_remove_field(Document *doc, const char *fname);
bool doc_delete_fields(Document *doc, const char *fname);
char *doc_to_s(Document *doc);
#endif
