#include "global.h"
#include "except.h"

bool except_show_pos = true;

char * const UNSUPPORTED_ERROR_MSG = "Unsupported operation";
char * const EOF_ERROR_MSG = "Read past end of file";

static thread_key_t exception_stack_key;
static thread_once_t exception_stack_key_once = THREAD_ONCE_INIT;

void exception_stack_alloc()
{
  thread_key_create(&exception_stack_key, NULL);
}

void xpush_context(xcontext_t *context)
{
  xcontext_t *top_context;
  thread_once(&exception_stack_key_once, *exception_stack_alloc);
  top_context = thread_getspecific(exception_stack_key); 
  context->next = top_context;
  thread_setspecific(exception_stack_key, context);
  context->handled = true;
  context->in_finally = false;
}

static inline void xraise_context(xcontext_t *context, int excode, char *msg)
{
  context->msg = msg;
  context->excode = excode;
  context->handled = false;
  longjmp(context->jbuf, excode);
}

void xraise(int excode, char *msg)
{
  xcontext_t *top_context;
  top_context = thread_getspecific(exception_stack_key); 
  if (!top_context) {
    eprintf(EXCEPTION_CODE, "Error: exception %d not handled: %s", excode, msg);
  } else if (!top_context->in_finally) {
    xraise_context(top_context, excode, msg);
  } else if (top_context->handled) {
    top_context->msg = msg;
    top_context->excode = excode;
    top_context->handled = false;
  }
}

void xpop_context()
{
  xcontext_t *top_context, *context;
  thread_once(&exception_stack_key_once, *exception_stack_alloc);
  top_context = thread_getspecific(exception_stack_key); 
  context = top_context->next;
  thread_setspecific(exception_stack_key, context);
  if (!top_context->handled) {
    xraise_context(context, top_context->excode, top_context->msg);
  }
}
