#include "search.h"
#include <string.h>

/***************************************************************************
 *
 * Filter
 *
 ***************************************************************************/

void filt_destroy(Filter *self)
{
  h_destroy(self->cache);
  free(self->name);
  free(self);
}

BitVector *filt_get_bv(Filter *self, IndexReader *ir)
{
  CacheObject *co = h_get(self->cache, ir);
  if (!co) {
    BitVector *bv;
    if (!ir->cache) {
      ir_add_cache(ir);
    }
    bv = self->get_bv(self, ir);
    co = co_create(self->cache, ir->cache, self, ir,
        (free_ft)&bv_destroy, (void *)bv);
  }
  return (BitVector *)co->obj;
}

char *filt_to_s_i(Filter *self)
{
  return estrdup(self->name);
}

uint filt_hash_dflt(Filter *self)
{
  return 0;
}

int filt_eq_dflt(Filter *self, Filter *o)
{
  return false;
}

Filter *filt_create(char *name)
{
  Filter *self = ALLOC(Filter);
  self->cache = co_hsh_create();
  self->name = estrdup(name);
  self->to_s = &filt_to_s_i;
  self->hash = &filt_hash_dflt;
  self->eq = &filt_eq_dflt;
  self->destroy = &filt_destroy;
  return self;
}

uint filt_hash(Filter *self)
{
  return str_hash(self->name) ^ self->hash(self);
}

int filt_eq(Filter *self, Filter *o)
{
  return ((self == o) ||
      ((strcmp(self->name, o->name) == 0) &&
       (self->eq == o->eq) &&
       (self->eq(self, o))));
}

/***************************************************************************
 *
 * QueryFilter
 *
 ***************************************************************************/

char *qfilt_to_s(Filter *self)
{
  QueryFilter *qf = (QueryFilter *)self->data;
  Query *query = (Query *)qf->query;
  char *query_str = query->to_s(query, "");
  char *filter_str = strfmt("QueryFilter< %s >", query_str);
  free(query_str);
  return filter_str;
}

BitVector *qfilt_get_bv(Filter *filt, IndexReader *ir)
{
  BitVector *bv = bv_create_size(ir->max_doc(ir));
  QueryFilter *qfilt = (QueryFilter *)filt->data;
  Searcher *sea = sea_create(ir);
  Weight *weight = q_weight(qfilt->query, sea);
  Scorer *scorer = weight->scorer(weight, ir);
  if (scorer) {
    while (scorer->next(scorer)) {
      bv_set(bv, scorer->doc);
    }
    scorer->destroy(scorer);
  }
  weight->destroy(weight);
  free(sea);
  return bv;
}

uint qfilt_hash(Filter *self)
{
  Query *query = (Query *)((QueryFilter *)self->data)->query;
  return q_hash(query);
}

int qfilt_eq(Filter *self, Filter *o)
{
  Query *query1 = (Query *)((QueryFilter *)self->data)->query;
  Query *query2 = (Query *)((QueryFilter *)o->data)->query;
  return q_eq(query1, query2);
}

void qfilt_destroy(Filter *self)
{
  Query *query = (Query *)((QueryFilter *)self->data)->query;
  q_deref(query);
  free(self->data);
  filt_destroy(self);
}

Filter *qfilt_create(Query *query)
{
  Filter *self;
  QueryFilter *qfilt = ALLOC(QueryFilter);
  qfilt->query = query;

  self = filt_create("QueryFilter");
  self->data = qfilt;
  self->get_bv = &qfilt_get_bv;
  self->hash = &qfilt_hash;
  self->eq = &qfilt_eq;
  self->to_s = &qfilt_to_s;
  self->destroy = &qfilt_destroy;
  return self;
}
