#ifndef FRT_HASH_H
#define FRT_HASH_H

#include "global.h"

#define NUM_ENTRIES 256
#define MULTIPLIER 31

typedef struct HashEntry {
  char      *name;
  void      *value;
  struct HashEntry *next;
} HashEntry;

HashEntry **ht_create();
int ht_count(HashEntry **ht);
void ht_destroy(HashEntry **ht);
void ht_destroy_all(HashEntry **ht, void (*fn)(void *));
void ht_set(HashEntry **ht, char *name, void *value);
void *ht_get(HashEntry **ht, char *name);
void *ht_delete(HashEntry **ht, char *name);

/****************************************************************************
 *
 * HshTable
 *
 ****************************************************************************/

#define Hsh_MINSIZE 8
#define SLOW_DOWN 50000 /* stop increasing the hash table so quickly to
                         * conserve memory */
extern char *dummy_key;
enum {
  HASH_KEY_DOES_NOT_EXIST = 0,
  HASH_KEY_SAME = 1,
  HASH_KEY_EQUAL = 2
};

typedef struct {
	int hash;      /* cached hash code of key */
  void *key;
	void *value;
} HshEntry;

typedef struct HshTable {
	int fill;  /* # Active + # Dummy */
	int used;  /* # Active */
	int mask;  /* size of table - 1 */

	/* table points to smalltable for small tables, else to
	 * additional malloc'ed memory. */
	HshEntry *table;
	HshEntry smalltable[Hsh_MINSIZE];
  HshEntry *(*lookup)(struct HshTable *ht, register const void *key);
  unsigned int (*hash)(const void *key);
  int (*eq)(const void *key1, const void *key2);
  free_ft free_key;
  free_ft free_value;
} HshTable;

typedef unsigned int (*hash_ft)(const void *key);
typedef int (*eq_ft)(const void *key1, const void *key2);

HshTable *h_new_str(free_ft free_key, free_ft free_value);
HshTable *h_new(hash_ft hash, eq_ft eq, free_ft free_key, free_ft free_value);
void h_destroy(HshTable *ht);
void h_clear(HshTable *ht);

void *h_get(HshTable *ht, const void *key);
int h_del(HshTable *ht, const void *key);
void *h_rem(HshTable *ht, const void *key, bool del_key);
int h_set(HshTable *ht, const void *key, void *value);
int h_set_safe(HshTable *ht, const void *key, void *value);
int h_has_key(HshTable *ht, const void *key);
unsigned int str_hash(const char *const str);

void h_each(HshTable *ht,
    void (*each_kv)(void *key, void *value, void *arg),
    void *arg);

typedef void *(*h_clone_func_t)(void *val);
HshTable *h_clone(HshTable *ht,
    h_clone_func_t clone_key,
    h_clone_func_t clone_value);

void dummy_free(void *p);
HshEntry *h_lookup_str(HshTable *ht, register const void *key_p);

#endif
