#include <hashset.h>
#include <string.h>
#define HS_MIN_SIZE 4

int *imalloc(int i)
{
  int *ip = ALLOC(int);
  *ip = i;
  return ip;
}

void hs_dummy_free(void *p){}

HashSet *hs_create(unsigned int (*hash)(const void *p),
    int (*eq)(const void *p1, const void *p2),
    void (*free_elem)(void *p))
{
  HashSet *hs = ALLOC(HashSet);
  hs->ht = h_new(hash, eq, NULL, &free);
  hs->elems = NULL;
  hs->capa = hs->size = 0;
  if (free_elem == NULL)
    hs->free_elem = &hs_dummy_free;
  else
    hs->free_elem = free_elem;
  return hs;
}

HashSet *hs_str_create(void (*free_elem)(void *p))
{
  HashSet *hs = ALLOC(HashSet);
  hs->ht = h_new_str((free_ft)NULL, &free);
  hs->elems = NULL;
  hs->capa = hs->size = 0;
  if (free_elem == NULL)
    hs->free_elem = &hs_dummy_free;
  else
    hs->free_elem = free_elem;
  return hs;
}

void hs_destroy(HashSet *hs)
{
  h_destroy(hs->ht);
  free(hs->elems);
  free(hs);
}

void hs_clear(HashSet *self)
{
  int i;
  for (i = self->size - 1; i >= 0; i--)
    hs_del(self, self->elems[i]);
}

void hs_destroy_all(HashSet *hs)
{
  int i;
  if (hs->free_elem != &dummy_free)
    for (i = 0; i < hs->size; i++)
      hs->free_elem(hs->elems[i]);
  hs_destroy(hs);
}

int hs_add(HashSet *hs, void *elem)
{
  int has_elem = h_has_key(hs->ht, elem);
  //printf("has_elem = %d %d:%d\n", has_elem, HASH_KEY_EQUAL, HASH_KEY_SAME);
  if (has_elem == HASH_KEY_EQUAL) {
    // We don't want to keep two of the same elem so free if necessary
    hs->free_elem(elem);
  } else if (has_elem == HASH_KEY_SAME) {
    // No need to do anything
  } else {
    // add the elem to the array, resizing if necessary
    if (hs->size >= hs->capa) {
      if (hs->capa == 0)
        hs->capa = HS_MIN_SIZE;
      else
        hs->capa *= 2;
      REALLOC_N(hs->elems, void *, hs->capa);
    }
    hs->elems[hs->size] = elem;
    h_set(hs->ht, elem, imalloc(hs->size));
    hs->size++;
  }
  return has_elem;
}

int hs_del(HashSet *hs, void *elem)
{
  void *tmp_elem = hs_rem(hs, elem);
  if (tmp_elem != NULL) {
    hs->free_elem(tmp_elem);
    return 1;
  } else {
    return 0;
  }
}

void *hs_rem(HashSet *hs, void *elem)
{
  void *ret_elem;
  int *index = (int *)h_get(hs->ht, elem);
  if (index == NULL) {
    return NULL;
  } else {
    int i = *index;
    ret_elem = hs->elems[i];
    h_del(hs->ht, elem);
    hs->size--;
    memmove(&hs->elems[i], &hs->elems[i+1], sizeof(void *) * (hs->size - i));
    return ret_elem;
  }
}

int hs_exists(HashSet *hs, void *elem)
{
  return h_has_key(hs->ht, elem);
}

HashSet *hs_merge(HashSet *hs, HashSet *other)
{
  int i;
  for (i = 0; i < other->size; i++) {
    hs_add(hs, other->elems[i]);
  }
  // Now free the other hashset. It is no longer needed. No need, however, to
  // delete the elements as they are in the new hash set
  hs_destroy(other);
  return hs;
}

void *hs_orig(HashSet *hs, void *elem)
{
  int *i = h_get(hs->ht, elem);
  if (i) return hs->elems[*i];
  else return NULL;
}
