#include "search.h"
#include <string.h>

/***************************************************************************
 *
 * Weight
 *
 ***************************************************************************/

char *maw_to_s(Weight *self)
{
  return strfmt("MatchAllWeight(%f)", self->value);
}

Explanation *maw_explain(Weight *self, IndexReader *ir, int doc_num)
{
  Explanation *expl;
  if (!ir->is_deleted(ir, doc_num)) {
    expl = expl_create(self->value, estrdup("MatchAllQuery: product of:"));
    expl_add_detail(expl, expl_create(self->query->boost, estrdup("boost")));
    expl_add_detail(expl, expl_create(self->qnorm, estrdup("query_norm")));
  } else {
    expl = expl_create(self->value,
        strfmt("MatchAllQuery: doc %d was deleted", doc_num));
  }

  return expl;
}

Weight *maw_create(Query *query, Searcher *searcher)
{
  Weight *self = w_create(query);

  self->scorer    = &masc_create;
  self->explain   = &maw_explain;
  self->to_s      = &maw_to_s;
  self->sum_of_squared_weights = &w_sum_of_squared_weights;

  self->similarity = query->get_similarity(query, searcher);
  self->idf = 1.0;

  return self;
}

/***************************************************************************
 *
 * MatchAllQuery
 *
 ***************************************************************************/

char *maq_to_s(Query *self, char *field)
{
  if (self->boost == 1.0) {
    return estrdup("MatchAll");
  } else {
    return strfmt("MatchAll^%f", self->boost);
  }
}

static uint maq_hash(Query *self)
{
  return 0;
}

static int maq_eq(Query *self, Query *o)
{
  return true;
}

Query *maq_create()
{
  Query *self = q_create();

  self->type = MATCH_ALL_QUERY;
  self->to_s = &maq_to_s;
  self->hash = &maq_hash;
  self->eq = &maq_eq;
  self->destroy_i = &q_destroy_i;
  self->create_weight_i = &maw_create;

  return self;
}

/***************************************************************************
 *
 * MatchAllScorer
 *
 ***************************************************************************/

float masc_score(Scorer *self)
{
  return ((MatchAllScorer *)self->data)->score;
}

bool masc_next(Scorer *self)
{
  MatchAllScorer *mas = (MatchAllScorer *)self->data;
  while (self->doc < (mas->max_doc - 1)) {
    self->doc++;
    if (!mas->ir->is_deleted(mas->ir, self->doc)) {
      return true;
    }
  }
  return false;
}

bool masc_skip_to(Scorer *self, int doc_num)
{
  self->doc = doc_num - 1;
  return masc_next(self);
}

Explanation *masc_explain(Scorer *self, int doc_num)
{
  return expl_create(1.0, estrdup("MatchAllScorer"));
}

Scorer *masc_create(Weight *weight, IndexReader *ir)
{
  Scorer *self = scorer_create(weight->similarity);
  MatchAllScorer *mas = ALLOC(MatchAllScorer);
  mas->ir = ir;
  mas->max_doc = ir->max_doc(ir);
  mas->score = weight->value;
  self->data = mas;

  self->doc = -1;
  self->score = &masc_score;
  self->next = &masc_next;
  self->skip_to = &masc_skip_to;
  self->explain = &masc_explain;
  self->destroy = &scorer_destroy_i;
  return self;
}
