#include <string.h>
#include "search.h"

/****************************************************************************
 *
 * PrefixQuery
 *
 ****************************************************************************/

char *prq_to_s(Query *self, char *field) 
{
  char *buffer, *bptr;
  Term *term = (Term *)self->data;
  size_t tlen = strlen(term->text);
  size_t flen = strlen(term->field);
  bptr = buffer = ALLOC_N(char, tlen + flen + 35);

  if (strcmp(term->field, field) != 0) {
    sprintf(bptr, "%s:", term->field);
    bptr += strlen(bptr);
  }
  sprintf(bptr, "%s*", term->text);
  if (self->boost != 1.0) {
    *bptr = '^';
    dbl_to_s(++bptr, self->boost);
  }

  return buffer;
}

Query *prq_rewrite(Query *self, IndexReader *ir)
{
  Term *prefix = (Term *)self->data;
  TermEnum *te = ir->terms_from(ir, prefix);
  char *prefix_text = prefix->text;
  size_t prefix_length = strlen(prefix_text);
  char *prefix_field = prefix->field;
  Query *tq;
  Query *bq = bq_create(true);

  TRY
    do { 
      TermBuffer *tb = te->tb_curr;
      if (!tb || strcmp(tb->field, prefix_field) != 0 ||
        strncmp(tb->text, prefix_text, prefix_length) != 0) {
        break;
      }
      tq = tq_create(term_create(tb->field, tb->text)); /* found a match */
      tq->boost = self->boost;                          /* set the boost */
      bq_add_query(bq, tq, BC_SHOULD);                  /* add to query  */
    } while (te->next(te));
  XFINALLY
    te->close(te);
  XENDTRY

  return bq;
}

static void prq_destroy(Query *self)
{
  if (self->destroy_all) term_destroy((Term *)self->data);
  q_destroy_i(self);
}

static uint prq_hash(Query *self)
{
  return term_hash((Term *)self->data);
}

static int prq_eq(Query *self, Query *o)
{
  return term_eq((Term *)self->data, (Term *)o->data);
}

Query *prefixq_create(Term *prefix)
{
  Query *self = q_create();
  self->data = prefix;

  self->type = PREFIX_QUERY;
  self->rewrite = &prq_rewrite;
  self->to_s = &prq_to_s;
  self->hash = &prq_hash;
  self->eq = &prq_eq;
  self->destroy_i = &prq_destroy;
  self->create_weight_i = &q_create_weight_unsup;

  return self;
}
