#include <string.h>
#include "search.h"

/****************************************************************************
 *
 * WildCardQuery
 *
 ****************************************************************************/

char *wcq_to_s(Query *self, char *field) 
{
  char *buffer, *bptr;
  Term *term = (Term *)self->data;
  size_t tlen = strlen(term->text);
  size_t flen = strlen(term->field);
  bptr = buffer = ALLOC_N(char, tlen + flen + 35);

  if (strcmp(term->field, field) != 0) {
    sprintf(bptr, "%s:", term->field);
    bptr += strlen(term->field) + 1;
  }
  sprintf(bptr, "%s", term->text);
  bptr = buffer + strlen(buffer);
  if (self->boost != 1.0) {
    *bptr = '^';
    dbl_to_s(++bptr, self->boost);
  }

  return buffer;
}

bool wc_match(char *pattern, char *text)
{
  char *p = pattern, *t = text, *xt; 

  /* include '\0' as we need to match empty string */
  char *text_last = t + strlen(t);

  for (;; p++, t++) {

    /* end of text so make sure end of pattern doesn't matter */
    if (*t == '\0') {
      while (*p) {
        if (*p != WILD_STRING) return false;
        p++;
      }
      return true;
    }

    /* If we've gone past the end of the pattern, return false. */
    if (*p == '\0') return false;

    /* Match a single character, so continue. */
    if (*p == WILD_CHAR) continue;

    if (*p == WILD_STRING) {
      // Look at the character beyond the '*'.
      p++;
      // Examine the string, starting at the last character.
      for (xt = text_last; xt >= t; xt--) {
        if (wc_match(p, xt)) return true;
      }
      return false;
    }
    if (*p != *t)
      return false;
  }

  return false;
}

Query *wcq_rewrite(Query *self, IndexReader *ir)
{
  Query *q;
  Query *tq;

  Term *term = (Term *)self->data;
  char *text = term->text;
  char *field = term->field;
  char *first_star = strchr(text, WILD_STRING);
  char *first_ques = strchr(text, WILD_CHAR);

  if (first_star == NULL && first_ques == NULL) {
    q = tq_create(term_clone(term));
  } else {
    TermEnum *te;
    Term prefix_term;
    char *prefix = NULL;

    char *pattern = (first_ques && (!first_star || (first_star > first_ques)))
      ? first_ques : first_star;

    int prefix_len = (int)(pattern - text);

    prefix_term.field = field;
    prefix_term.text = (char *)EMPTY_STRING;
    if (prefix_len > 0) {
      prefix = ALLOC_N(char, prefix_len + 1);
      strncpy(prefix, text, prefix_len);
      prefix_term.text = prefix;
      prefix_term.text[prefix_len] = '\0';
    }
    te = ir->terms_from(ir, &prefix_term);

    q = bq_create(true);
    if (te) {
      TermBuffer *tb = te->tb_curr;
      do { 
        if (strcmp(tb->field, field) != 0 ||
            (prefix && strncmp(tb->text, prefix, prefix_len) != 0))
          break;

        if (wc_match(pattern, tb->text + prefix_len)) {
          tq = tq_create(term_create(tb->field, tb->text)); /* found match */
          tq->boost = self->boost;                          /* set boost */
          bq_add_query(q, tq, BC_SHOULD);                   /* add query */
        }
      } while ((tb = te->next(te)) != NULL);
      te->close(te);
    }
    free(prefix);
  }

  return q;
}

static void wcq_destroy(Query *self)
{
  if (self->destroy_all) term_destroy((Term *)self->data);
  q_destroy_i(self);
}

static uint wcq_hash(Query *self)
{
  return term_hash((Term *)self->data);
}

static int wcq_eq(Query *self, Query *o)
{
  return term_eq((Term *)self->data, (Term *)o->data);
}

Query *wcq_create(Term *term)
{
  Query *self = q_create();

  self->data = term;

  self->type = WILD_CARD_QUERY;
  self->rewrite = &wcq_rewrite;
  self->to_s = &wcq_to_s;
  self->hash = &wcq_hash;
  self->eq = &wcq_eq;
  self->destroy_i = &wcq_destroy;
  self->create_weight_i = &q_create_weight_unsup;

  return self;
}
