#ifndef FRT_SIMILARITY_H
#define FRT_SIMILARITY_H

typedef struct Searcher Searcher;

/****************************************************************************
 *
 * Term
 *
 ****************************************************************************/

typedef struct Term {
  char *field;
  char *text;
} Term;

Term *term_clone(Term *term);
Term *term_create(const char *field, char *text);
void term_destroy(Term *self);
int term_cmp(void *t1, void *t2);
int term_eq(const void *t1, const void *t2);
unsigned int term_hash(const void *t);
char *term_to_s(Term *term);

/***************************************************************************
 *
 * Similarity
 *
 ***************************************************************************/

typedef struct Similarity Similarity;

struct Similarity {
  void *data;
  float norm_table[256];
  float (*length_norm)(Similarity *self, char *field, int num_terms);
  float (*query_norm)(Similarity *self, float sum_of_squared_weights);
  float (*tf)(Similarity *self, float freq);
  float (*sloppy_freq)(Similarity *self, int distance);
  float (*idf_term)(Similarity *self, Term *term, Searcher *searcher);
  float (*idf_phrase)(Similarity *self, Term **terms,
      int tcnt, Searcher *searcher);
  float (*idf)(Similarity *self, int doc_freq, int num_docs);
  float (*coord)(Similarity *self, int overlap, int max_overlap);
  float (*decode_norm)(Similarity *self, uchar b);
  uchar (*encode_norm)(Similarity *self, float f);
  void  (*destroy)(Similarity *self);
};

#define sim_length_norm(msim, field, num_terms) msim->length_norm(msim, field, num_terms)
#define sim_query_norm(msim, sosw) msim->query_norm(msim, sosw)
#define sim_tf(msim, freq) msim->tf(msim, freq)
#define sim_sloppy_freq(msim, distance) msim->sloppy_freq(msim, distance)
#define sim_idf_term(msim, term, searcher) msim->idf_term(msim, term, searcher)
#define sim_idf_phrase(msim, terms, tcnt, searcher) msim->idf_phrase(msim, terms, tcnt, searcher)
#define sim_idf(msim, doc_freq, num_docs) msim->idf(msim, doc_freq, num_docs)
#define sim_coord(msim, overlap, max_overlap) msim->coord(msim, overlap, max_overlap)
#define sim_decode_norm(msim, b) msim->decode_norm(msim, b)
#define sim_encode_norm(msim, f) msim->encode_norm(msim, f)
#define sim_destroy(msim) msim->destroy(msim)

float byte_to_float(uchar b);
uchar float_to_byte(float f);

Similarity *sim_create_default();

#endif
